﻿using SewingDressesContracts.BindingModels;
using SewingDressesContracts.ViewModels;
using SewingDressesDataModels.Enums;
using SewingDressesDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace SewingDressesDatabaseImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        [Required]
        public int DressId { get; private set; }
        [Required]
        public int ClientId { get; private set; }
        public int? ImplementId { get; private set; }
        [Required]
        public int Count { get; private set; }
        [Required]
        public double Sum { get; private set; }
        [Required]
        public OrderStatus Status { get; private set; }
        [Required]
        public DateTime DateCreate { get; private set; } = DateTime.Now;
        public DateTime? DateImplement { get; private set; }
        public virtual Dress Dress { get; set; }
        public virtual Client Client { get; set; }
        public virtual Implement? Implement { get; set; }

        public static Order? Create(OrderBindingModel model, SewingDressesDatabase context)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                DressId = model.DressId,
                ClientId = model.ClientId,
                ImplementId = model.ImplementId,
                Client = context.Clients.FirstOrDefault(x => x.Id == model.ClientId)!,
                Dress = context.Dresses.FirstOrDefault(x => x.Id == model.DressId)!,
                Implement = model.ImplementId.HasValue ? context.Implements.FirstOrDefault(x => x.Id == model.ImplementId) : null,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate
            };
        }
        public void Update(OrderBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            ImplementId = model.ImplementId;
            DateImplement = model.DateImplement;
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            DressId = DressId,
            DressName = Dress.DressName,
            ClientId = ClientId,
            ClientFIO = Client.ClientFIO,
            ImplementId = ImplementId,
            ImplementFIO = Implement?.ImplementFIO,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };
    }
}
