﻿using SewingDressesContracts.BindingModels;
using SewingDressesContracts.ViewModels;
using SewingDressesDataModels.Enums;
using SewingDressesDataModels.Models;
using System.Data;
using System.Xml.Linq;

namespace SewingDressesFileImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        public int DressId { get; private set; }
        public int ClientId { get; private set; }
        public int? ImplementId { get; private set; }
        public int Count { get; private set; }
        public double Sum { get; private set; }
        public OrderStatus Status { get; private set; }
        public DateTime DateCreate { get; private set; } = DateTime.Now;
        public DateTime? DateImplement { get; private set;}

        public static Order? Create(OrderBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                DressId = model.DressId,
                ClientId = model.ClientId,
                ImplementId = model.ImplementId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate
            };
        }
        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            var dateImplement = element.Element("DateImplement")?.Value;
            return new Order()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                DressId = Convert.ToInt32(element.Element("DressId")!.Value),
                ClientId = Convert.ToInt32(element.Element("ClientId")!.Value),
                ImplementId = Convert.ToInt32(element.Element("ImplementId")?.Value),
				Count = Convert.ToInt32(element.Element("Count")!.Value),
                Sum = Convert.ToDouble(element.Element("Sum")!.Value, new System.Globalization.CultureInfo("en-US")),
                Status = (OrderStatus)Convert.ToInt32(element.Element("Status")!.Value),
                DateCreate = Convert.ToDateTime(element.Element("DateCreate")?.Value),
                DateImplement = string.IsNullOrEmpty(dateImplement) ? null : Convert.ToDateTime(dateImplement)
            };
        }
        public void Update(OrderBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            DressId = DressId,
            ClientId = ClientId,
            ImplementId = ImplementId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };
        public XElement GetXElement => new("Order",
            new XAttribute("Id", Id),
            new XElement("DressId", DressId),
			new XElement("ClientId", ClientId),
            new XElement("ImplementId", ImplementId),
			new XElement("Count", Count),
            new XElement("Sum", Sum),
            new XElement("Status", Status - OrderStatus.Принят),
            new XElement("DateCreate", DateCreate),
            new XElement("DateImplement", DateImplement));
    }
}
